﻿using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Panel.Controllers;
using System;
using System.Collections.Generic;
using System.IdentityModel.Tokens.Jwt;
using System.Linq;
using System.Security.Claims;
using System.Threading.Tasks;
using WebApiServer.Authentication;
using WebApiServer.ViewModels;

namespace WebApiServer.Controllers
{
    public class AccountController : Controller
    {
        private readonly UserManager<ApplicationUser> _userManager;
        private readonly SignInManager<ApplicationUser> _signInManager;
        private readonly ApplicationDbContext dbContext;

        public AccountController(UserManager<ApplicationUser> userManager, SignInManager<ApplicationUser> signInManager, ApplicationDbContext dbContext)
        {
            _userManager = userManager;
            _signInManager = signInManager;
            this.dbContext = dbContext;
        }
        public IActionResult Index(int skip = 0, int take = 50)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var query = dbContext.Users.OrderBy(r => r.Id).ToList();

                ViewData["NextSkip"] = skip + take;
                ViewData["PrevSkip"] = skip - take;

                var rows = query.Skip(skip).Take(take).Select(r => new UserViewModel()
                {
                    UserId = r.Id,
                    UserName = r.UserName,
                    DisplayName = r.DisplayName,
                    FCM = r.FCM,
                    CreatedDate = r.CreatedDate,
                }).ToList();

                return View(rows);
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }

        [HttpGet]
        [AllowAnonymous]
        public IActionResult Login()
        {
            return View();
        }
        [HttpPost]
        [AllowAnonymous]
        public async Task<IActionResult> Login(LoginViewModel userModel)
        {
            if (!ModelState.IsValid)
            {
                ModelState.AddModelError(string.Empty, "Invalid Login Attempt");
                return View(userModel);
            }
            var user = await _userManager.FindByNameAsync(userModel.Username);
            if (user != null)
            {
                if (await _userManager.CheckPasswordAsync(user, userModel.Password))
                {
                    var identity = new ClaimsIdentity(IdentityConstants.ApplicationScheme);
                    identity.AddClaim(new Claim(ClaimTypes.NameIdentifier, user.Id));
                    identity.AddClaim(new Claim(ClaimTypes.Name, user.UserName));
                    var userRoles = await _userManager.GetRolesAsync(user);
                    if (userRoles.Contains("Administrator"))
                    {
                        foreach (var userRole in userRoles)
                        {
                            identity.AddClaim(new Claim(ClaimTypes.Role, userRole));
                        }
                        await HttpContext.SignInAsync(IdentityConstants.ApplicationScheme,
                            new ClaimsPrincipal(identity));

                        HttpContext.Session.SetString("Username", user.UserName);
                        if (userRoles.Count > 0)
                            HttpContext.Session.SetString("Role", userRoles[0]);
                        else
                            HttpContext.Session.Remove("Role");

                        return RedirectToAction(nameof(HomeController.Index), "Home");
                    }
                }
            }
            ModelState.AddModelError("", "Invalid UserName or Password");
            return View();
        }
        [HttpGet]
        [AllowAnonymous]
        public IActionResult ChangePassword()
        {
            return View();
        }

        [HttpPost]
        public async Task<IActionResult> ChangePassword(ChangePasswordModel userModel)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                ApplicationUser user = await _userManager.FindByNameAsync(username);
                if (user != null)
                {
                    if (userModel.Password == userModel.Password2)
                    {
                        user.PasswordHash = _userManager.PasswordHasher.HashPassword(user, userModel.Password);
                        var result = await _userManager.UpdateAsync(user);
                        if (result.Succeeded)
                        {
                            TempData["Message"] = "Passwords changed";
                        }
                    }
                    else
                    {
                        TempData["Message"] = "Passwords are not the same";
                    }
                }
            }
            return View();
        }

        public async Task<IActionResult> Logout()
        {
            await HttpContext.SignOutAsync();

            return RedirectToAction("Login");
        }


        [Route("EditSources")]
        public async Task<IActionResult> EditSources(string id)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var sources = await (from sItem in dbContext.Sources
                               from userSource in dbContext.UserSources.Where(o => o.UserId == id && o.SourceId == sItem.Id).DefaultIfEmpty()
                                select new EditSourcesViewModel.SourceViewModel()
                             {
                                 SourceId = sItem.Id,
                                 SourceName = sItem.Name,
                                 Value = userSource != null ? userSource.Value : 0,
                             }).OrderBy(r => r.SourceId).ToListAsync();

                if (sources != null)
                {
                    var model = new EditSourcesViewModel
                    {
                        UserId = id,
                        Sources = sources,
                    };
                    return View(model);
                }
                return RedirectToAction("Index", controllerName: "Account");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }


        [HttpPost]
        [Route("SaveSources")]
        public async Task<IActionResult> SaveSources([FromForm] EditSourcesViewModel model)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                if (model.Sources != null && model.Sources.Count > 0)
                {
                    for (int i=0;i< model.Sources.Count;i++)
                    {
                        var source = model.Sources[i];
                        var sourceID = source.SourceId;
                        var s = await dbContext.UserSources.FirstOrDefaultAsync(r => r.UserId == model.UserId && r.SourceId == sourceID);
                        if (s != null)
                        {
                            s.Value = source.Value;
                        }
                        else
                        { 
                            dbContext.UserSources.Add(new Models.UserSource()
                            {
                                UserId = model.UserId,
                                SourceId = source.SourceId,
                                Value = source.Value,
                            });

                        }
                    }

                    await dbContext.SaveChangesAsync();
                    TempData["Message"] = "Sources successfully saved.";
                }
                return RedirectToAction("Index", controllerName: "Account");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }
    }
}
