﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Routing;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Caching.Memory;
using Microsoft.Extensions.Configuration;
using Microsoft.Net.Http.Headers;
using Newtonsoft.Json;
using Panel.Controllers;
using WebApiServer.Authentication;
using WebApiServer.Models;
using WebApiServer.ViewModels;

namespace WebApiServer.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class EventController : Controller
    {
        private readonly UserManager<ApplicationUser> userManager;
        private readonly IConfiguration configuration;
        private readonly ApplicationDbContext dbContext;
        private readonly IMemoryCache memoryCache;
        private readonly IHttpClientFactory _httpClientFactory;

        public enum EventStatus
        {
            NotStarted = 0,
            Started = 1,
            Processing = 2,
            Finished = 3,
        }

        public EventController(UserManager<ApplicationUser> userManager, IConfiguration configuration, ApplicationDbContext dbContext, IMemoryCache memoryCache, IHttpClientFactory httpClientFactory)
        {
            this.userManager = userManager;
            this.configuration = configuration;
            this.dbContext = dbContext;
            this.memoryCache = memoryCache;
            this._httpClientFactory = httpClientFactory;
        }

        [Authorize]
        [HttpGet]
        [Route("GetAll")]
        public async Task<IActionResult> GetEvents()
        {
            var events = await (from ev in dbContext.Events
                                join banner in dbContext.Files on ev.BannerId equals banner.Id
                                where ev.Id != 1 && (ev.Status == (int)EventStatus.Started || (ev.Status == (int)EventStatus.Finished && ev.EndDate <= DateTime.UtcNow.AddHours(5)))
             select new Event
             {
                 Id = ev.Id,
                 BannerId = ev.BannerId,
                 Banner = banner.Id + banner.Extension,
                 Metadata = ev.Metadata,
                 Title = ev.Title,
                 StartDate = ev.StartDate,
                 EndDate = ev.EndDate,
                 Status = ev.Status,
             }).Take(50).ToListAsync();

            for (int i = 0; i < events.Count; i++)
            {
                var eventInfo = events[i];
                if (eventInfo.Status == (int)EventStatus.Started && eventInfo.EndDate <= DateTime.UtcNow)
                {
                    FinishEvent(eventInfo.Id);
                    eventInfo.Status = (int)EventStatus.Finished;
                }
                
                if (eventInfo.Status == (int)EventStatus.Finished)
                {
                    var users = await (from ev in dbContext.UserCompletedEvents
                                       join user in dbContext.Users on ev.UserId equals user.Id
                                       where ev.Id == eventInfo.Id
                                       select new Event.EventWinnerInfo
                                       {
                                           UserId = user.Id,
                                           DisplayName = user.DisplayName,
                                           Metadata = user.Metadata,
                                           Rank = ev.Rank,
                                       }).ToListAsync();

                    eventInfo.Winners = users;
                }
            }
            return Ok(events);
        }

        [Authorize]
        [HttpGet]
        [Route("Get")]
        public async Task<IActionResult> GetEvent(long eventId)
        {
            var eventInfo = await GetEventDetail(eventId);
            if (eventInfo == null)
                return BadRequest();

            return Ok(eventInfo);
        }
        private async Task<Event> GetEventDetail(long eventId)
        {
            var eventInfo = await (from ev in dbContext.Events
                                   join banner in dbContext.Files on ev.BannerId equals banner.Id
                                   where ev.Id == eventId
                                   select new Event
                                   {
                                       Id = ev.Id,
                                       BannerId = ev.BannerId,
                                       Banner = banner.Id + banner.Extension,
                                       Metadata = ev.Metadata,
                                       Title = ev.Title,
                                       StartDate = ev.StartDate,
                                       EndDate = ev.EndDate,
                                   }).FirstOrDefaultAsync();
            if (eventInfo == null)
                return null;

            if (eventInfo.Status == (int)EventStatus.Started && eventInfo.EndDate <= DateTime.UtcNow)
            {
                FinishEvent(eventInfo.Id);
                eventInfo.Status = (int)EventStatus.Processing;
            }
            else if (eventInfo.Status == (int)EventStatus.Finished)
            {
                var users = await (from ev in dbContext.UserCompletedEvents
                                   join user in dbContext.Users on ev.UserId equals user.Id
                                   where ev.Id == eventId
                                   select new Event.EventWinnerInfo
                                   {
                                       UserId = user.Id,
                                       DisplayName = user.DisplayName,
                                       Metadata = user.Metadata,
                                       Rank = ev.Rank,
                                   }).ToListAsync();

                eventInfo.Winners = users;
            }
            return eventInfo;
        }

        [Authorize]
        [HttpGet]
        [Route("GetLevels")]
        public async Task<IActionResult> GetEventLevels(long eventId)
        {
            var eventInfo = await dbContext.Events.FirstOrDefaultAsync(e => e.Id == eventId);
            if (eventInfo != null)
            {
                var eventLevels = await (from ev in dbContext.EventLevels
                                         join bannerFile in dbContext.Files on ev.BannerId equals bannerFile.Id
                                         join bundle in dbContext.Files on ev.AssetBundleFileId equals bundle.Id into ps
                                         from bundlep in ps.DefaultIfEmpty()
                                         where ev.EventId == eventInfo.Id
                                         select new EventLevel
                                         {
                                             Id = ev.Id,
                                             EventId = ev.EventId,
                                             LevelId = ev.LevelId,
                                             Title = ev.Title,
                                             IsCheckpoint = ev.IsCheckpoint,
                                             Points = ev.Points,
                                             Metadata = ev.Metadata,
                                             AssetBundle = bundlep != null ? (bundlep.Id + bundlep.Extension) : "",
                                             Banner = bannerFile.Id + bannerFile.Extension,
                                             Portrait = ev.Portrait,
                                         }).OrderBy(i=>i.Id).Take(50).ToListAsync();
                return Ok(eventLevels);
            }
            return StatusCode(StatusCodes.Status500InternalServerError);
        }


        [Authorize]
        [HttpGet]
        [Route("CompletedLevels")]
        public async Task<IActionResult> GetCompletedLevels(long eventId)
        {
            ApplicationUser user = await userManager.FindByNameAsync(HttpContext.User.Identity.Name);

            var eventDetail = await dbContext.Events.Select(e=>new {Id = e.Id}).FirstOrDefaultAsync(e => e.Id == eventId);
            if (eventDetail != null)
            {
                bool isExist = memoryCache.TryGetValue(user.Id + "_" + eventId, out var levelId);
                if (isExist)
                {
                    await FailEvent(new Request_LevelComplete()
                    {
                        EventId = eventId,
                        LevelId = (long)levelId,
                    });
                }


                var items = await dbContext.EventPoints.Join(dbContext.EventLevels,
                 lp => lp.LevelId,
                 l => l.Id,
                 (l, lp) => new { points = l, level = lp })
                .Where(lp => lp.points.EventId == eventDetail.Id && lp.points.UserId == user.Id && lp.points.Point > 0)
                .OrderByDescending(l => l.points.Point)
                .GroupBy(l => l.level.LevelId)
                .Select(ssa => ssa.Key).ToListAsync();

                return Ok(items);
            }
            return StatusCode(StatusCodes.Status500InternalServerError);
        }


        [Authorize]
        [HttpGet]
        [Route("GetPoints")]
        public async Task<IActionResult> GetPoints(long eventId, int skip = 0, int take = 20)
        {
            var eventInfo = await dbContext.Events.Select(e => new { Id = e.Id, Status = e.Status }).FirstOrDefaultAsync(e => e.Id == eventId);
            if (eventInfo != null)
            {
                var items = await dbContext.EventPoints.Join(dbContext.Events,
             lp => lp.EventId,
             l => l.Id,
             (l, lp) => new { points = l, evInfo = lp })
             .Join(dbContext.Users,
             lp => lp.points.UserId,
             l => l.Id,
             (l, lp) => new { detail = l, user = lp })
            .Where(lp => lp.detail.evInfo.Id == eventId)
            .GroupBy(l => new { l.detail.points.UserId, l.user.DisplayName, l.user.Metadata })
            .Select(ssa => new Response_EventPointItem
            {
                UserId = ssa.Key.UserId,
                DisplayName = ssa.Key.DisplayName,
                Metadata = ssa.Key.Metadata,
                Point = ssa.Sum(s => s.detail.points.Point),
                Rank = 0,
                MaxLevelId = ssa.Max(s => s.detail.points.LevelId),
            })
            .OrderByDescending(l => l.Point)
            .Skip(skip).Take(20)
            .ToListAsync();

                var itemsCount = await dbContext.EventPoints.Join(dbContext.Events,
                 lp => lp.EventId,
                 l => l.Id,
                 (l, lp) => new { points = l, leaderboard = lp })
                 .Join(dbContext.Users,
                 lp => lp.points.UserId,
                 l => l.Id,
                 (l, lp) => new { detail = l, user = lp })
                .Where(lp => lp.detail.leaderboard.Id == eventId)
                .GroupBy(l => new { l.detail.points.UserId, l.user.UserName, l.user.Metadata })
                .CountAsync();

                for (int i = 0; i < items.Count; i++)
                {
                    var item = items[i];
                    item.Rank = i + 1 + skip;
                }

                return Ok(new
                {
                    items = items,
                    itemCount = itemsCount,
                    eventStatus = eventInfo.Status,
                });
            }
            return StatusCode(StatusCodes.Status500InternalServerError);
        }

        [Authorize]
        [HttpPost]
        [Route("StartLevel")]
        public async Task<IActionResult> StartLevel([FromBody] Request_LevelComplete model)
        {
            ApplicationUser user = await userManager.FindByNameAsync(HttpContext.User.Identity.Name);
            if (user != null)
            {
                var eventInfo = await dbContext.Events.Select(e=> new {Id = e.Id}).FirstOrDefaultAsync(e => e.Id == model.EventId);
                if (eventInfo != null)
                {
                    var eventLevels = await (from ev in dbContext.EventLevels where ev.EventId == eventInfo.Id
                                             select new {id = ev.Id , LevelId = ev.LevelId }).OrderBy(i => i.LevelId).ToListAsync();

                    var index = eventLevels.FindIndex(m => m.LevelId == model.LevelId);
                    if (index > -1)
                    {
                        if (index > 0)
                        {
                            var lastLevel = eventLevels[index-1];
                            var id = lastLevel.id;
                            var points = await dbContext.EventPoints.Where(p => p.UserId == user.Id && p.EventId == model.EventId && p.LevelId == id).ToListAsync();
                            if (points.Count <= 0)
                            {
                                return StatusCode(StatusCodes.Status500InternalServerError);
                            }
                        }

                        var cacheEntryOptions = new MemoryCacheEntryOptions().SetSlidingExpiration(TimeSpan.FromMinutes(30));
                        memoryCache.Set(user.Id + "_" + model.EventId, model.LevelId, cacheEntryOptions);
                        return Ok();
                    }
                }
            }
            return StatusCode(StatusCodes.Status500InternalServerError);
        }

        [Authorize]
        [HttpPost]
        [Route("CompleteLevel")]
        public async Task<IActionResult> CompleteLevel([FromBody] Request_LevelComplete model)
        {
            ApplicationUser user = await userManager.FindByNameAsync(HttpContext.User.Identity.Name);

            var eventDetail = await dbContext.Events.Select(e => new { Id = e.Id, StartDate = e.StartDate, EndDate = e.EndDate, Status = e.Status }).FirstOrDefaultAsync(e => e.Id == model.EventId);
            if (eventDetail != null && eventDetail.Status == (int)EventStatus.Started)
            {
                if (eventDetail.StartDate <= DateTime.UtcNow && eventDetail.EndDate > DateTime.UtcNow)
                {
                    if (user != null)
                    {
                        bool isExist = memoryCache.TryGetValue(user.Id + "_" + model.EventId, out var levelName);
                        if (!isExist)
                        {
                            await FailEvent(model);
                            return StatusCode(StatusCodes.Status500InternalServerError);
                        }

                        var levelDetail = await dbContext.EventLevels.FirstOrDefaultAsync(l => l.EventId == eventDetail.Id && l.LevelId == model.LevelId);
                        if (levelDetail != null)
                        {
                            bool isLastLevel = false;
                            var eventLevels = await (from ev in dbContext.EventLevels where ev.EventId == model.EventId
                                                     select ev.Id).OrderBy(i => i).ToListAsync();
                            var index = eventLevels.IndexOf(model.LevelId);
                            if (index == eventLevels.Count -1)
                                isLastLevel = true;
                               
                            
                            if (await dbContext.EventLevels.Where(l => l.EventId == eventDetail.Id && l.LevelId < levelDetail.LevelId).CountAsync() > 0)
                            {
                            }
                            else
                            {
                                var firstLevelId = await dbContext.EventLevels.Where(l => l.EventId == eventDetail.Id).MinAsync(l => l.Id);
                                if (levelDetail.Id != firstLevelId)
                                    return StatusCode(StatusCodes.Status500InternalServerError);
                            }

                            memoryCache.Remove(user.Id + "_" + model.EventId);

                            using var transaction = dbContext.Database.BeginTransaction();

                            var eventPointDetail = await dbContext.EventPoints.FirstOrDefaultAsync(l => l.EventId == eventDetail.Id && l.LevelId == levelDetail.Id && l.UserId == user.Id && l.Point > 0);
                            if (eventPointDetail != null)
                            {
                                eventPointDetail.Point = levelDetail.Points;
                                eventPointDetail.Duration = model.Duration;
                                eventPointDetail.Logs = model.Logs;
                            }
                            else
                            {
                                var levelInfo = new EventPointItem()
                                {
                                    UserId = user.Id,
                                    EventId = eventDetail.Id,
                                    LevelId = levelDetail.Id,
                                    Point = levelDetail.Points,
                                    Duration = model.Duration,
                                    Logs = model.Logs,
                                };
                                dbContext.EventPoints.Add(levelInfo);
                            }

                            var r = await dbContext.SaveChangesAsync();
                            
                            if (isLastLevel)
                            {
                                dbContext.UserCompletedEvents.Add(new UserCompletedEvent()
                                {
                                    UserId = user.Id,
                                    EventId = eventDetail.Id,
                                    Rank = 1,
                                });
                                var result = await dbContext.SaveChangesAsync();
                                if (result > 0)
                                {
                                    transaction.Commit();
                                    return Ok(GetEventDetail(model.EventId));
                                }
                                else
                                {
                                    await transaction.RollbackAsync();
                                }
                            }
                            else
                            {
                                transaction.Commit();
                                return Ok();
                            }
                        }
                    }
                }
            }
            else if (eventDetail.Status == (int)EventStatus.Finished)
            {
                return Ok(GetEventDetail(model.EventId));
            }
            return StatusCode(StatusCodes.Status500InternalServerError);
        }

        [Authorize]
        [HttpPost]
        [Route("FailLevel")]
        public async Task<IActionResult> FailEvent([FromBody] Request_LevelComplete model)
        {
            ApplicationUser user = await userManager.FindByNameAsync(HttpContext.User.Identity.Name);

            var eventDetail = await dbContext.Events.Select(e => new { Id = e.Id, StartDate=e.StartDate,EndDate=e.EndDate }).FirstOrDefaultAsync(e => e.Id == model.EventId);
            if (eventDetail != null)
            {
                if (eventDetail.StartDate <= DateTime.UtcNow && eventDetail.EndDate > DateTime.UtcNow)
                {
                    if (user != null)
                    {
                        memoryCache.Remove(user.Id + "_" + model.EventId);
                        var levelDetail = await dbContext.EventLevels.FirstOrDefaultAsync(l => l.EventId == eventDetail.Id && l.LevelId == model.LevelId);
                        if (levelDetail != null)
                        {
                            try
                            {
                                var lastLevelId = await dbContext.EventLevels.Where(l => l.EventId == eventDetail.Id && l.LevelId < levelDetail.Id && l.IsCheckpoint == true).MaxAsync(l => l.Id);

                                int result = dbContext.Database.ExecuteSqlRaw("Update RS_EventPoints set Point = 0 where EventId = " + eventDetail.Id + " and LevelId > " + lastLevelId);

                            }
                            catch (Exception ex)
                            {

                            }
                            return Ok();
                        }
                    }
                }
            }
            return StatusCode(StatusCodes.Status500InternalServerError);
        }
        [Authorize]
        [HttpGet]
        [Route("CompletedEvents")]
        public async Task<IActionResult> GetCompletedEvents()
        {
            ApplicationUser user = await userManager.FindByNameAsync(HttpContext.User.Identity.Name);
            var completedEvents = await dbContext.UserCompletedEvents.Where(e => e.UserId == user.Id && e.Status == (int)UserCompletedEvent.StatusEnum.None).ToListAsync();
            return Ok(completedEvents);
        }

        [Authorize]
        [HttpPost]
        [Route("CollectEventPrize")]
        public async Task<IActionResult> CollectEventPrize([FromBody] Request_CollectEventPrize model)
        {
            ApplicationUser user = await userManager.FindByNameAsync(HttpContext.User.Identity.Name);

            var completedEventInfo = await dbContext.UserCompletedEvents.FirstOrDefaultAsync(e => e.UserId == user.Id && e.EventId == model.EventId);
            if (completedEventInfo != null)
            {
                var eventInfo = await dbContext.Events.FirstOrDefaultAsync(e => e.Id == completedEventInfo.EventId);
                if (eventInfo != null)
                {
                    completedEventInfo.Status = (int)UserCompletedEvent.StatusEnum.Collecting;
                    if (await dbContext.SaveChangesAsync() > 0)
                    {
                        completedEventInfo = await dbContext.UserCompletedEvents.FirstOrDefaultAsync(e => e.UserId == user.Id && e.EventId == model.EventId);

                        completedEventInfo.Status = (int)UserCompletedEvent.StatusEnum.Collected;
                        if (await dbContext.SaveChangesAsync() > 0)
                        {
                            return Ok();
                        }
                    }
                }
            }
            return StatusCode(StatusCodes.Status500InternalServerError);
        }

        public IActionResult Index(int skip = 0, int take = 50)
        {
            var request = HttpContext.Request;
            var _baseURL = $"{request.Scheme}://{request.Host}";
            ViewData["BaseUrl"] = _baseURL;

            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var query = dbContext.Events.OrderBy(r => r.Id).ToList();//.Where(e => e.Id != 1)

                ViewData["NextSkip"] = skip + take;
                ViewData["PrevSkip"] = skip - take;

                var rows = query.Skip(skip).Take(take).Join(
                    dbContext.Files,
                    entryPoint => entryPoint.BannerId,
                    entry => entry.Id,
                    (entryPoint, entry) => new EventViewModel
                    {
                        Id = entryPoint.Id,
                        Title = entryPoint.Title,
                        Banner = entry.Id + entry.Extension,
                        Metadata = entryPoint.Metadata,
                        StartDate = entryPoint.StartDate,
                        EndDate = entryPoint.EndDate,
                        Status = ((EventStatus)entryPoint.Status),
                    }
                ).ToList();

                return View(rows);
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }
        [Route("Edit")]
        public async Task<IActionResult> Edit(long id)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var ev = await dbContext.Events.FirstOrDefaultAsync(r => r.Id == id);
                if (ev != null)
                {
                    var banner = await dbContext.Files.FirstOrDefaultAsync(r => r.Id == ev.BannerId);
                    if (banner != null)
                    {
                        var bundles = new List<FileModel>();
                        var images = await dbContext.Files.Where(f => f.FileType == (int)FileController.FileType.Image).ToListAsync();
                        if (images != null)
                        {
                            return View(new NewEventViewModel
                            {
                                Event = new EventViewModel()
                                {
                                    Id = ev.Id,
                                    Title = ev.Title,
                                    BannerId = ev.BannerId,
                                    Banner = banner.Id + banner.Extension,
                                    Metadata = ev.Metadata,
                                    StartDate = ev.StartDate,
                                    EndDate = ev.EndDate,
                                },
                                Images = images,
                            });
                        }
                    }
                }
                return RedirectToAction("Index", controllerName: "Event");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }
        [Route("New")]
        public async Task<IActionResult> New()
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var bundles = await dbContext.Files.Where(f => f.FileType == (int)FileController.FileType.AssetBundle).ToListAsync();
                var images = await dbContext.Files.Where(f => f.FileType == (int)FileController.FileType.Image).ToListAsync();
                if (bundles != null && images != null)
                    return View(new NewEventViewModel
                    {
                        Event = new EventViewModel()
                        {
                            StartDate = DateTime.Now,
                            EndDate = DateTime.Now.AddDays(1),
                        },
                        Images = images
                    });
                return RedirectToAction("Index", controllerName: "Event");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }

        [HttpPost]
        [Route("Save")]
        public async Task<IActionResult> Save([FromForm] NewEventViewModel model)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var ev = await dbContext.Events.FirstOrDefaultAsync(r => r.Id == model.Event.Id);
                if (ev != null)
                {
                    if (model.Event.Id > 1)
                    {
                        ev.Title = model.Event.Title;
                        ev.StartDate = model.Event.StartDate;
                        ev.EndDate = model.Event.EndDate;
                        ev.BannerId = model.Event.BannerId;
                    }
                    int result = await dbContext.SaveChangesAsync();
                    if (result > 0)
                    {
                        TempData["Message"] = "Event successfully saved.";
                    }
                }
                return RedirectToAction("Index", controllerName: "Event");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }

        [HttpPost]
        [Route("SaveNew")]
        public async Task<IActionResult> SaveNew([FromForm] NewEventViewModel model)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                dbContext.Events.Add(new Event()
                {
                    Title = model.Event.Title,
                    BannerId = model.Event.BannerId,
                    StartDate = model.Event.StartDate,
                    EndDate = model.Event.EndDate,
                    Metadata = model.Event.Metadata,
                    Status = 0,
                });
                int result = await dbContext.SaveChangesAsync();
                if (result > 0)
                {
                    TempData["Message"] = "Event successfully saved.";
                }
                return RedirectToAction("Index", controllerName: "Event");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }

        [Route("Delete")]
        public async Task<IActionResult> Delete(int id)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var ev = await dbContext.Events.Where(x => x.Id == id).FirstOrDefaultAsync();
                if (ev == null) return null;

                dbContext.Events.Remove(ev);

                if (dbContext.SaveChanges() > 0)
                {

                }
            }
            return RedirectToAction("Index", controllerName: "Event");
        }

        [Route("EditLevels")]
        public async Task<IActionResult> EditLevels(long eventId)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var ev = await dbContext.Events.FirstOrDefaultAsync(r => r.Id == eventId);
                if (ev != null)
                {
                    var levels = await dbContext.EventLevels.Where(l=>l.EventId == ev.Id).OrderBy(l=>l.LevelId).ToListAsync();
                    if (levels != null)
                    {
                        var model = new EventLevelsViewModel
                        {
                            Event = new EventViewModel()
                            {
                                Id = ev.Id,
                                Title = ev.Title,
                                BannerId = ev.BannerId,
                                Metadata = ev.Metadata,
                                StartDate = ev.StartDate,
                                EndDate = ev.EndDate,
                            },
                            AssetBundles = await dbContext.Files.Where(b => b.FileType == (int)FileController.FileType.AssetBundle).ToListAsync(),
                            Images = await dbContext.Files.Where(b => b.FileType == (int)FileController.FileType.Image).ToListAsync(),
                            Levels = levels.Select(l => new EventLevelViewModel()
                            {
                                Id = l.Id,
                                EventId = l.EventId,
                                LevelId = l.LevelId,
                                Title = l.Title,
                                IsCheckpoint = l.IsCheckpoint,
                                Metadata = l.Metadata,
                                Points = l.Points,
                                AssetBundleFileId = l.AssetBundleFileId,
                                BannerFileId = l.BannerId,
                                Portrait = l.Portrait,
                            }).ToList(),
                        };
                        return View(model);
                    }
                }
                return RedirectToAction("Index", controllerName: "Event");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }
        [Route("NewLevel")]
        public async Task<IActionResult> NewLevel(long eventId)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var bundles = await dbContext.Files.Where(f => f.FileType == (int)FileController.FileType.AssetBundle).ToListAsync();
                var images = await dbContext.Files.Where(f => f.FileType == (int)FileController.FileType.Image).ToListAsync();
                if (bundles != null && images != null)
                    return View(new NewEventLevelViewModel
                    {
                        EventLevel = new EventLevelViewModel()
                        {
                            EventId = eventId,
                        },
                        Bundles = bundles,
                        Images = images
                    });
                return RedirectToAction("Index", controllerName: "Event");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }
        [HttpPost]
        [Route("SaveNewLevel")]
        public async Task<IActionResult> SaveNewLevel([FromForm] NewEventLevelViewModel model)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                dbContext.EventLevels.Add(new EventLevel()
                {
                    Title = model.EventLevel.Title,
                    EventId = model.EventLevel.EventId,
                    LevelId = model.EventLevel.LevelId,
                    AssetBundleFileId = model.EventLevel.AssetBundleFileId,
                    BannerId = model.EventLevel.BannerFileId,
                    IsCheckpoint = model.EventLevel.IsCheckpoint,
                    Metadata = model.EventLevel.Metadata,
                    Points = model.EventLevel.Points,
                    Portrait= model.EventLevel.Portrait,
                });
                int result = await dbContext.SaveChangesAsync();
                if (result > 0)
                {
                    TempData["Message"] = "Event Level successfully saved.";
                }
                return Redirect(Request.Headers["Referer"].ToString());
                //return RedirectToAction("EventLevels", controllerName: "Event");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }
        [Route("DeleteLevel")]
        public async Task<IActionResult> DeleteLevel(int id)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var ev = await dbContext.EventLevels.Where(x => x.Id == id).FirstOrDefaultAsync();
                if (ev == null) return null;

                dbContext.EventLevels.Remove(ev);

                if (dbContext.SaveChanges() > 0)
                {
                    return Redirect(Request.Headers["Referer"].ToString());
                }
            }
            return RedirectToAction("Index", controllerName: "Event");
        }


        [HttpPost]
        [Route("SaveLevels")]
        public async Task<IActionResult> SaveLevels([FromForm] EventLevelsViewModel model)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var levelIds = model.Levels.Select(l => l.Id).ToList();
                var levels = await dbContext.EventLevels.Where(l => levelIds.Contains(l.Id)).ToListAsync();
                if (model.Levels.Count == levels.Count)
                {
                    int i = 0;
                    foreach (var level in levels)
                    {
                        level.AssetBundleFileId = model.Levels[i].AssetBundleFileId;
                        level.BannerId = model.Levels[i].BannerFileId;
                        level.LevelId = model.Levels[i].LevelId;
                        level.Title = model.Levels[i].Title;
                        level.Points = model.Levels[i].Points;
                        level.Metadata = model.Levels[i].Metadata;
                        level.IsCheckpoint = model.Levels[i].IsCheckpoint;
                        level.Portrait= model.Levels[i].Portrait;
                        i++;
                    }
                    dbContext.SaveChanges();
                }
                return RedirectToAction("Index", controllerName: "Event");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }

        [Route("Start")]
        public async Task<IActionResult> Start(long id)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var ev = await dbContext.Events.FirstOrDefaultAsync(r => r.Id == id);
                if (ev != null)
                {
                    if (ev.Status == (int)EventStatus.NotStarted)
                    {
                        ev.Status = (int)EventStatus.Started;
                        dbContext.SaveChanges();
                        TempData["Message"] = "Event successfully started.";
                    }
                }
                return RedirectToAction("Index", controllerName: "Event");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }
        [Route("Stop")]
        public async Task<IActionResult> Stop(long id)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var ev = await dbContext.Events.FirstOrDefaultAsync(r => r.Id == id);
                if (ev != null)
                {
                    if (ev.Status == (int)EventStatus.Started)
                    {
                        ev.Status = (int)EventStatus.NotStarted;
                        dbContext.SaveChanges();
                        TempData["Message"] = "Event successfully started.";
                    }
                }
                return RedirectToAction("Index", controllerName: "Event");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }

        private async void FinishEvent(long eventId)
        {
            if (memoryCache.TryGetValue("Finish_" + eventId, out var data))
                return;
            var cacheEntryOptions = new MemoryCacheEntryOptions().SetSlidingExpiration(TimeSpan.FromMinutes(30));
            memoryCache.Set("Finish_" + eventId, true, cacheEntryOptions);

            var eventInfo = await dbContext.Events.FirstOrDefaultAsync(e => e.Id == eventId);

            if (eventInfo != null)
            {
                eventInfo.Status = (int)EventStatus.Finished;
                await dbContext.SaveChangesAsync();
            }

            memoryCache.Remove("Finish_" + eventId);
        }

        [Route("Leaderboard")]
        public async Task<IActionResult> Leaderboard(long id, int skip = 0, int take = 50)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var eventInfo = await dbContext.Events.Select(e => new { Id = e.Id, Status = e.Status }).FirstOrDefaultAsync(e => e.Id == id);
                if (eventInfo != null)
                {
                    var items = await dbContext.EventPoints.Join(dbContext.Events,
                     lp => lp.EventId,
                     l => l.Id,
                     (l, lp) => new { points = l, evInfo = lp })
                     .Join(dbContext.Users,
                     lp => lp.points.UserId,
                     l => l.Id,
                     (l, lp) => new { detail = l, user = lp })
                    .Where(lp => lp.detail.evInfo.Id == id)
                    .GroupBy(l => new { l.detail.points.UserId, l.user.DisplayName, l.user.Metadata })
                    .Select(ssa => new LeaderboardItemViewModel
                    {
                        UserId = ssa.Key.UserId,
                        DisplayName = ssa.Key.DisplayName,
                        Metadata = ssa.Key.Metadata,
                        Point = ssa.Sum(s => s.detail.points.Point),
                        Rank = 0,
                        MaxLevelId = ssa.Max(s => s.detail.points.LevelId),
                    })
                    .OrderByDescending(l => l.Point)
                    .Skip(skip).Take(20)
                    .ToListAsync();

                        var itemsCount = await dbContext.EventPoints.Join(dbContext.Events,
                         lp => lp.EventId,
                         l => l.Id,
                         (l, lp) => new { points = l, leaderboard = lp })
                         .Join(dbContext.Users,
                         lp => lp.points.UserId,
                         l => l.Id,
                         (l, lp) => new { detail = l, user = lp })
                        .Where(lp => lp.detail.leaderboard.Id == id)
                        .GroupBy(l => new { l.detail.points.UserId, l.user.UserName, l.user.Metadata })
                        .CountAsync();

                        for (int i = 0; i < items.Count; i++)
                        {
                            var item = items[i];
                            item.Rank = i + 1 + skip;
                    }
                    ViewData["NextSkip"] = skip + take;
                    ViewData["PrevSkip"] = skip - take;

                    return View(items);
                }
                return RedirectToAction("Login", controllerName: "Account");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }
    }
}
