﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using WebApiServer.Authentication;
using WebApiServer.Models;
using WebApiServer.ViewModels;

namespace Panel.Controllers
{
    public class FileController : Controller
    {
        private readonly ILogger<HomeController> _logger;
        private readonly ApplicationDbContext context;
        private readonly IConfiguration configuration;

        public enum FileType
        {
            None = 0,
            Image = 1,
            AssetBundle = 2
        }
        public FileController(ILogger<HomeController> logger, ApplicationDbContext context, IConfiguration configuration)
        {
            _logger = logger;
            this.context = context;
            this.configuration = configuration;
        }

        public string GetDataDirectory()
        {
            return $@"{AppDomain.CurrentDomain.BaseDirectory}";
        }

        public async Task<IActionResult> Index(int skip = 0, int take = 50)
        {
            var request = HttpContext.Request;
            var _baseURL = $"{request.Scheme}://{request.Host}";
            ViewData["BaseUrl"] = _baseURL;

            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role != null && role.Equals("Administrator"))
            {
                ViewData["NextSkip"] = skip + take;
                ViewData["PrevSkip"] = skip - take;

                ViewBag.Message = TempData["Message"];

                var model = new FileViewModel()
                {
                    Images = await context.Files.Where(f => f.FileType == (int)FileType.Image && f.Id > 1).ToListAsync()
                };
                return View(model);
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }
        public async Task<IActionResult> AssetBundles(int skip = 0, int take = 50)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role != null && role.Equals("Administrator"))
            {
                ViewData["NextSkip"] = skip + take;
                ViewData["PrevSkip"] = skip - take;

                ViewBag.Message = TempData["Message"];
                return View(new FileViewModel()
                {
                    Images = await context.Files.Where(f => f.FileType == (int)FileType.AssetBundle && f.Id > 2).ToListAsync()
                });
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }

        [HttpPost]
        [Route("UploadAssetBundle")]
        public async Task<IActionResult> UploadAssetBundle(IFormFile file, string name, string description)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var basePath = Path.Combine(GetDataDirectory() + "\\AssetBundles\\");//Directory.GetCurrentDirectory()
                bool basePathExists = System.IO.Directory.Exists(basePath);
                if (!basePathExists) Directory.CreateDirectory(basePath);
                var fileName = Path.GetFileNameWithoutExtension(file.FileName);
                var extension = Path.GetExtension(file.FileName);

                using var transaction = context.Database.BeginTransaction();
                try
                {
                    var fileModel = new FileModel
                    {
                        FileType = (int)FileType.AssetBundle,
                        Extension = extension,
                        Name = name,
                        Description = description,
                    };
                    context.Files.Add(fileModel);
                    int result = await context.SaveChangesAsync();
                    if (result > 0)
                    {
                        var filePath = Path.Combine(basePath, fileModel.Id + extension);
                        if (!System.IO.File.Exists(filePath))
                        {
                            using (var stream = new FileStream(filePath, FileMode.Create))
                            {
                                await file.CopyToAsync(stream);
                                transaction.Commit();
                                TempData["Message"] = "File successfully uploaded.";
                                return RedirectToAction("AssetBundles");
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    transaction.Rollback();
                    if (ex.InnerException.Message.Contains("duplicate"))
                        TempData["Message"] = "Duplicate Resource Name.";
                    return RedirectToAction("AssetBundles");
                }
                transaction.Rollback();
                return RedirectToAction("AssetBundles");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }
        public async Task<IActionResult> Delete(int id)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var imageBasePath = Path.Combine(GetDataDirectory() + "\\Images\\");
                var bundleBasePath = Path.Combine(GetDataDirectory() + "\\AssetBundles\\");
                var file = await context.Files.Where(x => x.Id == id).FirstOrDefaultAsync();
                if (file == null) return null;
                var filePath = Path.Combine(file.FileType == (int)FileType.Image ? imageBasePath : bundleBasePath, file.Id + file.Extension);
                if (System.IO.File.Exists(filePath))
                {
                    System.IO.File.Delete(filePath);
                }
                context.Files.Remove(file);
                context.SaveChanges();
                TempData["Message"] = $"Removed {file.Name + file.Extension} successfully.";
                return RedirectToAction(file.FileType == (int)FileType.Image ? "Index" : "AssetBundles");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }
        [HttpPost]
        [Route("UploadImage")]
        public async Task<IActionResult> UploadImage(IFormFile file, string description)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var basePath = Path.Combine(GetDataDirectory() + "\\Images\\");
                bool basePathExists = System.IO.Directory.Exists(basePath);
                if (!basePathExists) Directory.CreateDirectory(basePath);
                var fileName = Path.GetFileNameWithoutExtension(file.FileName);
                var extension = Path.GetExtension(file.FileName);

                using var transaction = context.Database.BeginTransaction();
                try
                {
                    var fileModel = new FileModel
                    {
                        ContentType = file.ContentType,
                        FileType = (int)FileType.Image,
                        Extension = extension,
                        Name = fileName,
                        Description = description
                    };
                    context.Files.Add(fileModel);
                    int result = await context.SaveChangesAsync();
                    if (result > 0)
                    {
                        var filePath = Path.Combine(basePath, fileModel.Id + extension);
                        if (!System.IO.File.Exists(filePath))
                        {
                            using (var stream = new FileStream(filePath, FileMode.Create))
                            {
                                await file.CopyToAsync(stream);
                                transaction.Commit();
                                TempData["Message"] = "File successfully uploaded.";
                                return RedirectToAction("Index");
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    transaction.Rollback();
                    return RedirectToAction("Index");
                }
                transaction.Rollback();
                return RedirectToAction("Index");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }
        public async Task<IActionResult> Download(int id)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var imageBasePath = Path.Combine(GetDataDirectory() + "\\Images\\");
                var bundleBasePath = Path.Combine(GetDataDirectory() + "\\AssetBundles\\");
                var file = await context.Files.Where(x => x.Id == id).FirstOrDefaultAsync();
                if (file == null) return null;
                var filePath = Path.Combine(file.FileType == (int)FileType.Image ? imageBasePath : bundleBasePath, file.Id + file.Extension);
                var memory = new MemoryStream();
                using (var stream = new FileStream(filePath, FileMode.Open))
                {
                    await stream.CopyToAsync(memory);
                }
                memory.Position = 0;
                return File(memory, file.ContentType, file.Name + file.Extension);
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }
    }
}
