﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Caching.Memory;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using Microsoft.Net.Http.Headers;
using Newtonsoft.Json;
using WebApiServer.Authentication;
using WebApiServer.Models;
using WebApiServer.ViewModels;

namespace WebApiServer.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class ShopItemsController : Controller
    {
        private readonly UserManager<ApplicationUser> userManager;
        private readonly IConfiguration configuration;
        private readonly ApplicationDbContext dbContext;
        private readonly IMemoryCache memoryCache;
        private readonly IHttpClientFactory _httpClientFactory;
        private ILogger _logger;
        public ShopItemsController(UserManager<ApplicationUser> userManager, IConfiguration configuration, ApplicationDbContext dbContext, IMemoryCache memoryCache, IHttpClientFactory httpClientFactory, ILoggerFactory loggerFactory)
        {
            this.userManager = userManager;
            this.configuration = configuration;
            this.dbContext = dbContext;
            this.memoryCache = memoryCache;
            this._httpClientFactory = httpClientFactory;
            _logger = loggerFactory.CreateLogger(typeof(TransactionController));
        }

        [Authorize]
        [HttpGet]
        [Route("GetAll")]
        public async Task<IActionResult> GetAll()
        {
            ApplicationUser user = await userManager.FindByNameAsync(HttpContext.User.Identity.Name);
            if (user != null)
            {
                var shopItems = await dbContext.ShopItems.OrderByDescending(t => t.Id).ToListAsync();
                return Ok(shopItems);
            }
            return Unauthorized();
        }

        public async Task<IActionResult> Index(int skip = 0, int take = 50)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var query = (from i in dbContext.ShopItems
                             join sItem in dbContext.Sources on i.SourceId equals sItem.Id
                             select new ShopItemViewModel
                             {
                                 Id = i.Id,
                                 Active = i.Active,
                                 Icon = i.Icon,
                                 Price = i.Price,
                                 Sku = i.Sku,
                                 SourceId = i.SourceId,
                                 SourceValue = i.SourceValue,
                                 Title = i.Title,
                                 SourceName = sItem.Name,
                             }).OrderBy(r => r.Id);

                ViewData["NextSkip"] = skip + take;
                ViewData["PrevSkip"] = skip - take;

                var rows = await query.Skip(skip).Take(take).ToListAsync();

                return View(rows);
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }


        [Route("Edit")]
        public async Task<IActionResult> Edit(long id)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var item = await dbContext.ShopItems.FirstOrDefaultAsync(r => r.Id == id);
                if (item != null)
                {
                    var source = await dbContext.Sources.FirstOrDefaultAsync(r => r.Id == item.SourceId);
                    if (source != null)
                    {
                        var sources = await dbContext.Sources.ToListAsync();
                        if (sources != null)
                        {
                            return View(new NewShopItemViewModel
                            {
                                Item = new ShopItemViewModel()
                                {
                                    Id = item.Id,
                                    Title = item.Title,
                                    Active = item.Active,
                                    Icon = item.Icon,
                                    Price = item.Price,
                                    Sku = item.Sku,
                                    SourceId = item.SourceId,
                                    SourceValue = item.SourceValue,
                                    SourceName = source.Name,
                                },
                                Sources = sources,
                            });
                        }
                    }
                }
                return RedirectToAction("Index", controllerName: "ShopItems");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }
        [Route("New")]
        public async Task<IActionResult> New()
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var sources = await dbContext.Sources.ToListAsync();
                if (sources != null)
                    return View(new NewShopItemViewModel
                    {
                        Item = new ShopItemViewModel()
                        {
                        },
                        Sources = sources
                    });
                return RedirectToAction("Index", controllerName: "ShopItems");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }

        [HttpPost]
        [Route("Save")]
        public async Task<IActionResult> Save([FromForm] NewShopItemViewModel model)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var item = await dbContext.ShopItems.FirstOrDefaultAsync(r => r.Id == model.Item.Id);
                if (item != null)
                {
                    item.Title = model.Item.Title;
                    item.Icon = model.Item.Icon;
                    item.Price = model.Item.Price;
                    item.Sku = model.Item.Sku;
                    item.SourceId = model.Item.SourceId;
                    item.SourceValue = model.Item.SourceValue;
                    int result = await dbContext.SaveChangesAsync();
                    if (result > 0)
                    {
                        TempData["Message"] = "Shop Item successfully saved.";
                    }
                }
                return RedirectToAction("Index", controllerName: "ShopItems");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }

        [HttpPost]
        [Route("SaveNew")]
        public async Task<IActionResult> SaveNew([FromForm] NewShopItemViewModel model)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                dbContext.ShopItems.Add(new ShopItem()
                {
                    Icon = model.Item.Icon,
                    Title = model.Item.Title,
                    Price = model.Item.Price,
                    Sku = model.Item.Sku,
                    SourceId = model.Item.SourceId,
                    SourceValue = model.Item.SourceValue,
                });
                int result = await dbContext.SaveChangesAsync();
                if (result > 0)
                {
                    TempData["Message"] = "Shop Item successfully saved.";
                }
                return RedirectToAction("Index", controllerName: "ShopItems");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }

        [Route("Delete")]
        public async Task<IActionResult> Delete(int id)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var item = await dbContext.ShopItems.Where(x => x.Id == id).FirstOrDefaultAsync();
                if (item == null) return null;

                dbContext.ShopItems.Remove(item);

                if (dbContext.SaveChanges() > 0)
                {

                }
            }
            return RedirectToAction("Index", controllerName: "ShopItems");
        }
        [Route("Start")]
        public async Task<IActionResult> Activate(long id)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var item = await dbContext.ShopItems.FirstOrDefaultAsync(r => r.Id == id);
                if (item != null)
                {
                    if (!item.Active)
                    {
                        item.Active = true;
                        dbContext.SaveChanges();
                        TempData["Message"] = "Shop Item successfully activated.";
                    }
                }
                return RedirectToAction("Index", controllerName: "ShopItems");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }
        [Route("Stop")]
        public async Task<IActionResult> Deactivate(long id)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var item = await dbContext.ShopItems.FirstOrDefaultAsync(r => r.Id == id);
                if (item != null)
                {
                    if (item.Active)
                    {
                        item.Active = false;
                        dbContext.SaveChanges();
                        TempData["Message"] = "Shop Item successfully deactivated.";
                    }
                }
                return RedirectToAction("Index", controllerName: "ShopItems");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }

    }
}
