﻿using System;
using System.Collections.Generic;
using System.IdentityModel.Tokens.Jwt;
using System.Linq;
using System.Security.Claims;
using System.Text;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.IdentityModel.Tokens;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using WebApiServer.Authentication;
using WebApiServer.Models;

namespace WebApiServer.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class UserController : ControllerBase
    {
        private readonly UserManager<ApplicationUser> userManager;
        private readonly RoleManager<IdentityRole> roleManager;
        private readonly IConfiguration configuration;
        private readonly ApplicationDbContext dbContext;

        public UserController(UserManager<ApplicationUser> userManager, RoleManager<IdentityRole> roleManager, IConfiguration configuration, ApplicationDbContext dbContext)
        {
            this.userManager = userManager;
            this.roleManager = roleManager;
            this.configuration = configuration;
            this.dbContext = dbContext;
        }

        [HttpPost]
        [Route("Login")]
        public async Task<IActionResult> Login([FromBody] Request_UserLogin model)
        {
            var user = await userManager.FindByNameAsync(model.DeviceId);
            if (user == null)
            {
                user = new ApplicationUser
                {
                    UserName = model.DeviceId,
                    Metadata = "{}",
                    SecurityStamp = Guid.NewGuid().ToString(),
                };
                var result = await userManager.CreateAsync(user, model.Password);

                if (!result.Succeeded)
                {
                    user = null;
                }
            }
            if (user != null && await userManager.CheckPasswordAsync(user, model.Password))
            {
                var userRoles = await userManager.GetRolesAsync(user);
                var authClaims = new List<Claim>
                {
                    new Claim(ClaimTypes.Name, model.DeviceId),
                    new Claim(JwtRegisteredClaimNames.Jti, Guid.NewGuid().ToString()),
                };
                foreach(var userRole in userRoles)
                {
                    authClaims.Add(new Claim(ClaimTypes.Role, userRole));
                }
                var authSigninKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(configuration["JWT:Secret"]));
                var token = new JwtSecurityToken(
                    issuer: configuration["JWT:ValidIssuer"],
                    audience: configuration["JWT:ValidAudience"],
                    expires: DateTime.Now.AddDays(100),
                    claims: authClaims,
                    signingCredentials: new SigningCredentials(authSigninKey, SecurityAlgorithms.HmacSha256)
                    );

                return Ok(new
                {
                    Id = user.Id,
                    DisplayName = user.DisplayName,
                    DeviceId = user.UserName,
                    Token = new JwtSecurityTokenHandler().WriteToken(token),
                    Metadata = user.Metadata,
                    FCM = user.FCM,
                });
            }
            return Unauthorized();
        }


        [Authorize]
        [HttpGet]
        [Route("GetSources")]
        public async Task<IActionResult> GetSources()
        {
            ApplicationUser user = await userManager.FindByNameAsync(HttpContext.User.Identity.Name);
            if (user != null)
            {
                var sources = await (from s in dbContext.Sources
                                join us in dbContext.UserSources on new { Id = s.Id, UserId = user.Id } equals new { Id = us.SourceId, UserId = us.UserId } into gj
                                from x in gj.DefaultIfEmpty()
                                     select new
                               {
                                   Name = s.Name,
                                   Value = (x == null ? 0 : x.Value)
                                     }).ToListAsync();

                return Ok(sources);
            }
            return Unauthorized();
        }

        [Authorize]
        [HttpPost]
        [Route("UseSource")]
        public async Task<IActionResult> UseSource([FromBody] Request_UseSource model)
        {
            ApplicationUser user = await userManager.FindByNameAsync(HttpContext.User.Identity.Name);
            if (user != null)
            {
                var sourceInfo = await dbContext.Sources.FirstOrDefaultAsync(e => e.Name == model.SourceName);
                if (sourceInfo != null)
                {
                    var userSource = await dbContext.UserSources.FirstOrDefaultAsync(e => e.SourceId == sourceInfo.Id && e.UserId == user.Id);
                    if (userSource != null)
                    {
                        if (userSource.Value >= model.Amount)
                        {
                            userSource.Value -= model.Amount;

                            await dbContext.SaveChangesAsync();
                            return Ok(new { });
                        }
                    }
                }
            }
            return StatusCode(StatusCodes.Status500InternalServerError);
        }

        [Authorize]
        [HttpPost]
        [Route("SetFCM")]
        public async Task<IActionResult> SetFCM([FromBody] Request_SetFCM model)
        {
            ApplicationUser user = await userManager.FindByNameAsync(HttpContext.User.Identity.Name);
            if (user != null)
            {
                user.FCM = model.Token;
                var result = await userManager.UpdateAsync(user);
                return Ok(new { });
            }
            return StatusCode(StatusCodes.Status500InternalServerError);
        }
    }
}
